// Crossing Gate Controller, version 1.10 Thu July 14, 2017 8:12AM
#include <PinChangeInt.h>
#include <Servo.h>  // servo library

// Note that this library disables PWM on
// pins 9 and 10!

Servo myservo1;  // servo control object
Servo myservo2;  // servo control object

/* This version is designed for the NANO.
   This code utilizes three PinChange Interrupts.

   1. On pin 2, the Start signal indicates the probable beginning of a train arrival.
   This ihterrupt routine first checks to see if the Disable pin is covered. If covered,
   this interrupt is ignored, because that indicates the train is leaving a gated block.  
   If not covered, it sets the START flag, for the loop() to use to set the GATE state, 
   and to set the delta t to then set the ARMS state.
 
   2. On pin 3, the Disable signal resets the START flag.
   
   3. On pin 4, the Stop signal indicates that both stop sensors are uncovered and 
   sets the STOP flag, for the loop() to use to stop the ARMS state and start the delta t
   to then stop the GATE state.

   All three of these interrupts are de-bounced 
   by setting/checking flags to indicate already serviced.

   4. Flasshers are on pins 5 and 6.  The Servos are on pins 9 and 10.

   5.    The photo transistors are short-circuit with light, open-circuit when covered.
    START trigger occurs when phototransistor becomes covered, therefore a RISING interrupt.
    Disable trigger is the same. The STOP trigger occurs when both stop detectors are uncovered,
    therefore a FALLING interrupt.
*/
#define STARTPIN 2
#define DISABLEPIN 3
#define STOPPIN 4
#define FLASHONE 5
#define FLASHTWO 6
#define GATEPIN 7
#define ARM1 9
#define ARM2 10
boolean volatile STARTFlag = false;
boolean volatile STOPDelayFlag = false;
boolean STOPFlag = false;
boolean GATEState = false;
boolean ArmUpState = false;
boolean ArmDownState = false;
boolean delayToArms = false;
boolean delayToEnd = false;
boolean whichFlash = false;
boolean sensorVal = false;
boolean FinishedDownFlag = false;
unsigned long startFlashMillis = 0;        // will store when we started flashing...
unsigned long currentFlashMillis = 0;
unsigned long startStopMillis = 0;        // will store when we started flashing...
unsigned long currentStopMillis = 0;
unsigned long startStartMillis =0;
unsigned long currentStartMillis =0;
unsigned long startDisableMillis =0;
unsigned long currentDisableMillis =0;
unsigned long startBellMillis = 0;        // will store when we started flashing...
unsigned long currentBellMillis = 0;
int  dots;
int servoCount1 = 0;
int servoCount2 = 0;
unsigned long startMillis = 0;        // will store when we started flashing...
unsigned long currentMillis = 0;
unsigned long startServoMillis = 0;        // will store when we started flashing...
unsigned long currentServoMillis = 0;
 // constants won't change :
const long interval = 3000;           // interval at which to delay arms (milliseconds)
const long flashInterval = 250;
const long servoInterval = 70;
const long bellInterval = 10000;
const byte gateOpenCount = 45;
const byte gateClosedCount = 0; //changed 10/5
// Set up the iterrupt routines
void pinStartfunc() { 
  if ( digitalRead( DISABLEPIN ) == LOW ) 
  {
    if ( !GATEState )  
    {
      GATEState = true;
      STARTFlag = true;
      Serial.println("Real Start");
    }
  }
}
void pinStopfunc() { 
  if (( GATEState ) && ( !STOPDelayFlag ))
  {
Serial.println("Stop");
    STOPDelayFlag = true;
    startStopMillis = millis();
  }
}
// end of interrupt routines
// we will call this routine each loop(), if and only if we have started flashing.
// we end calling this routine after we start arms coming down, then again after arms and before 
// terminating gate state entirely.

// used to determine delay before activating arms, after flashing starts, and continuing
// flashing after stop signal has been received.

void setup() {
  pinMode(STARTPIN, INPUT_PULLUP); digitalWrite(STARTPIN, HIGH);
  PCintPort::attachInterrupt(STARTPIN, &pinStartfunc, RISING); 
  pinMode(DISABLEPIN, INPUT_PULLUP); digitalWrite(DISABLEPIN, HIGH);
  // The pin change interrupt routine has been eliminated in 1.11
  pinMode(STOPPIN, INPUT_PULLUP); digitalWrite(STOPPIN, HIGH);
  PCintPort::attachInterrupt(STOPPIN, &pinStopfunc, FALLING);

  myservo1.attach(9); // Servo pin 9, located rear of tracks.
  myservo2.attach(10);  // Servo pin 10, located front of tracks.

  pinMode(GATEPIN, OUTPUT); digitalWrite(GATEPIN, HIGH);
  pinMode(FLASHONE, OUTPUT); digitalWrite(FLASHONE, LOW);
  pinMode(FLASHTWO, OUTPUT); digitalWrite(FLASHTWO, LOW);
  
  myservo1.write(0); // run both gate arms UP.
  myservo2.write(45);

  STARTFlag = false;
  STOPDelayFlag = false;
  STOPFlag = false;
  GATEState = false;
  ArmUpState = false;
  ArmDownState = false;
  delayToArms = false;
  delayToEnd = false;
  whichFlash = false;
  sensorVal = false;
  FinishedDownFlag = false;
  delay(1000);
  Serial.begin(9600);
  Serial.println("------Start Train Detection-CrossingGate-rev.1.11------");
}

void loop() {
//..................Stop delay and pin check............................................
  // first, see if the stop flag is real?
  if ( STOPDelayFlag )
  {
    currentStopMillis = millis();
    if ( currentStopMillis - startStopMillis > 200 )
    {
      if ( digitalRead( STOPPIN ) == LOW ) 
      {
Serial.println("Real Stop");
        STOPFlag = true;
        STOPDelayFlag = false;
      } else {    
         startStopMillis = millis();
      }

    }
  }
//..............................................................

// here, we just print out the character that what state we are in...
/*  if ( GATEState ) 
  { Serial.print("G ");
  } else { 
    Serial.print(".");
   } */
/*  if ( delayToArms ) Serial.print("T1 ");
  if ( ArmDownState ) Serial.print("D ");
  if ( delayToEnd ) Serial.print("T2 ");
  if ( ArmUpState ) Serial.print("U ");
  dots++;
  if ( dots >= 15 ){
    dots = 0;
    Serial.println();
  }*/
// First thing in the loop() is to look for a train?
  if ( STARTFlag == true) 
  {
    STARTFlag = false;
    startFlashMillis = millis();
    startBellMillis = millis();
    digitalWrite(GATEPIN, LOW); // start bell ringing
    
// here, we turn ON the Flashing lights and start a time delay to start the ArmDownState
    startMillis = millis();
    delayToArms = true;  
  }
// now, check each loop() to see if we are in the delay until ARMState should begin?
  if ( delayToArms == true ) 
  {
    currentMillis = millis();
    if (currentMillis - startMillis >= interval)
    {
      delayToArms = false;
      ArmDownState = true;
      startMillis = millis();
      servoCount1 = gateOpenCount;
      servoCount2 = gateClosedCount;
    }
  }
// now, check to see if we are in ArmDownState ?
  if ( ArmDownState == true )
  {
    currentMillis = millis();
    if (currentMillis - startMillis >= servoInterval)
    {
      if ( servoCount1-- <= gateClosedCount )
      { // Serial.println();
        ArmDownState = false;
        FinishedDownFlag = true;
// and continue with flashing lights only      
      } else { //Serial.print("DS, ");
//        myservo1.write(servoCount1);
        myservo2.write(servoCount1);
        myservo1.write(servoCount2++);
//        Serial.print(servoCount1); Serial.print(" ,");
//        Serial.print(servoCount2); Serial.print(" :");
        startMillis = millis();
      }
    }
  }
  if (( STOPFlag == true ) && (FinishedDownFlag == true ))
  {
    ArmUpState = true;
    STOPFlag = false;
    FinishedDownFlag = false;
// now, we start the time period to lift the arms.
    servoCount1 = gateClosedCount;
    servoCount2 = gateOpenCount;
    startMillis = millis();
  }
  if ( ArmUpState == true )
  {
    currentMillis = millis();
    if (currentMillis - startMillis >= servoInterval)
    {
     if ( servoCount1++ >= gateOpenCount )
      {
        ArmUpState = false;
        delayToEnd = true;  
        startMillis = millis();
// and continue with delay to end      
      } else { 
 //       myservo1.write(servoCount1);
        myservo2.write(servoCount1);
        myservo1.write(servoCount2--);
        startMillis = millis();
      }
    }
  }
// now, check each loop() to see if we are in the delay until ARMState should begin?
  if ( delayToEnd == true ) 
  {
    currentMillis = millis();
    if (currentMillis - startMillis >= interval)
    {
      delayToEnd = false;
      GATEState = false;
      Serial.println("End Gate");
    }
  }
    if ( GATEState ) 
// waggle CrossBuck LEDs during the entire Gate State
  {    currentFlashMillis = millis();
    if (currentFlashMillis - startFlashMillis >= flashInterval)
    {
      if ( whichFlash )
      {
        whichFlash = false;
        digitalWrite(FLASHONE, HIGH); 
        digitalWrite(FLASHTWO, LOW); 
      } else {
        whichFlash = true;
        digitalWrite(FLASHONE, LOW); 
        digitalWrite(FLASHTWO, HIGH); 
      }
      startFlashMillis = millis();
    }
    currentBellMillis = millis();
    if (currentBellMillis - startBellMillis >= bellInterval)
    {
 //     digitalWrite(GATEPIN, HIGH); 
      delay(10);
      digitalWrite(GATEPIN, LOW); 
      startBellMillis = millis();
    }
// Keep GATEPIN active (LOW) all during gate period.
  } else { digitalWrite(FLASHONE, HIGH); // Remember that the LED common is HIGH!
           digitalWrite(FLASHTWO, HIGH); 
           digitalWrite(GATEPIN, HIGH); 
         }
 // delay(25);
}
/* Changes from 1.4 to 1.5 include:
 *  1. changed servo count from 0 to 3 in setup and to 2 in up/dn sequence trying to stop agitation.
 *  2. attempting to use the STOP delay and pin check to tame the START and DISABLE  interrupts. Seems to work with STOP?
 *  also removed some #defs of unused stuff.
 *  
 *  Changes from 1.5 to 1.6 include:
 *  changing start and disable interrupts to be like the Stop function, using delays and checking pin state.
 */
