/*This file is intended to provide serial data transmission between the Toggle Panel
and the Turnout LED Panel and Turnout Actuators. Both channels of code herein utilize the same pin 
configuration, the only exception when the serial pin is changed from Output to Input.  
Otherwise all pin definiations and useages are identical.

Three pins are allocated to code control functions: Pin 2 to switch on/off the startup turnout exercise. 
Pin 13 will illuminate the internal LED indicating the status of Pin 2.  Pin 3 determines the state of
the scanning process.  
Both Pins 2 and 3 are set up so that when pulled LOW they enable their respective processes.
*/
boolean setUp = false;  // make true if you want to run through the turnout exercise?
boolean scanFlag = false;
byte setUpPin = 2;
byte scanPin = 3;
byte setupLED = 13;
byte channel0 = 7;
byte channel1 = 8;
byte clockPin = 9;
byte latchPin = 10;
byte serialPin = 11;
byte channel_A = 0;
byte channel_B = 1;
byte channel_C = 2;
byte channel_D = 3;
unsigned int dataReadIn = 0;
unsigned int previousData = 0;
unsigned int dataTemp;
int z = 0;
// Code run during 'setup()' will run only once at the beginning and never again unless 'reset'.
void setup() {
  //configure pin2 as an input and enable the internal pull-up resistor to test for 'setUp'.
  pinMode(setUpPin, INPUT_PULLUP);
  pinMode(scanPin, INPUT_PULLUP);
  pinMode(setupLED, OUTPUT);
  int sensorVal = digitalRead(setUpPin);
  if (sensorVal == HIGH) {
    digitalWrite(setupLED, LOW); setUp = false;
  } else {
    digitalWrite(setupLED, HIGH); setUp = true;
  }
  sensorVal = digitalRead(scanPin);
  if (sensorVal == HIGH) {
    scanFlag = false;
  } else {
    scanFlag = true;
  }
  Serial.begin(9600);
  Serial.println(".........TogglesToLEDs-11-18-16........................");
// Set all used pins to OUTPUT (serialPin will be set to INPUT to read in toggles.
  for( byte i = channel0; i <= serialPin; i++) {
    pinMode( i, OUTPUT);
  }  
/* serialPin will be OUTPUT when sending to turnout LED panels and turnout actuators.
   serialPin will be INPUT when receiving data from toggle switches and OCC detectors.
/* Here, I want to read the switches, invert the data, cause each Turnout to switch opposite,
  then, switch back, one at a time to the toggle switch status. I do this by inverting one bit
  at a time, and sending the data to the actuators, then sending again to bring turnout back to 
  toggle switch status.
  ONLY execute when setUp is set TRUE above*/
  if ( setUp == true )
  {
    pinMode(serialPin, INPUT);
    dataReadIn = shiftInWord( channel_D);
// Serial.print("Data Read In = "); Serial.println(dataReadIn, BIN);
// Serial.println(".................................");
   dataTemp = dataReadIn;
    pinMode(serialPin, OUTPUT);
    shiftOutWord( channel_A, dataTemp ); // Set up hardware with latest configuration.
    for ( byte i = 0; i <=16; i++ )
    {
      dataTemp ^= (1 << i );
// Serial.print("Modified Data = "); Serial.println(dataTemp, BIN);
      shiftOutWord( channel_A, dataTemp );
      delay(500);
    }
// Serial.println(".................................");
    delay(100);
    for ( byte i = 0; i <=16; i++ )
    {
      dataTemp ^= (1 << i );
// Serial.print("Re-Modified Data = "); Serial.println(dataTemp, BIN);
      shiftOutWord( channel_A, dataTemp );
      delay(500);
    }
  }
 } // End of Setup, run only once.

//.............................................................
void loop() {
// put my main code here, to run repeatedly:
// We will initially read in the Toggle switches status in a 16 bit word on the 4th (input) channel.
  pinMode(serialPin, INPUT);

  dataReadIn = shiftInWord( channel_D);
  if ( scanFlag == true ) previousData = 0;
  delayMicroseconds(100);
//  if ( dataReadIn != previousData )
//  {
    previousData = dataReadIn;  // refresh previous with new data.
// At this point we send the data to the Turnout actuators and Turnout LED panel.
  changeChannel(channel_B);
// Now, get ready to shift the data out to the LEDs and actuators.
      pinMode(serialPin, OUTPUT);

      shiftOutWord( channel_A, dataReadIn );
//  } // now, change to another channel for scope work.
  changeChannel(channel_B);
  delay(10);
} // End of Loop, now start subroutines:

//.............................................................
// This first routine shifts 16 bits IN from toggle switches.
unsigned int shiftInWord( byte channelNr ) 
{
  unsigned int dataRead = 0;
  unsigned int  temp = 0;
  //first, select the proper channel
  changeChannel(channelNr);
  delayMicroseconds(10);
  //write LOAD high (inverted logic) to enable parallel input
  digitalWrite(latchPin,LOW);
  delayMicroseconds(10);
  //write clock High to bring parallel data into shiftregister
  digitalWrite(clockPin,HIGH);
  delayMicroseconds(10);
 //write clock low to seal data
  digitalWrite(clockPin,LOW);
  delayMicroseconds(10);
 //Lower LOAD(inverted)to allow shifting to take place
  digitalWrite(latchPin,HIGH);
  delayMicroseconds(10);
  
  //now, use as many clocks as this channel needs
  for (int i = 15; i >=0; i--)
  {
    temp = digitalRead(serialPin);
    temp = !temp; // added to have sent data same as received?
    temp = ( temp << i );
    dataRead = dataRead | temp;
    delayMicroseconds(10);
    digitalWrite(clockPin,HIGH);
    delayMicroseconds(10);
    digitalWrite(clockPin,LOW);
    delayMicroseconds(10);
  }
  delayMicroseconds(10);
  return dataRead;
} // End shift IN 16 bits.
//.............................................................
// Next routine shifts 16 bytes out to the LEDs.
 void shiftOutWord( byte channelNr, unsigned int dataWord  ) 
{
  // first select the appropriate output channel
  changeChannel(channelNr);
  delayMicroseconds(10);
  //now, use as many clocks as this channel needs
  for (int i = 15; i>=0; i--)
  {
    if (dataWord & (1 << i )) {
      digitalWrite(serialPin, HIGH);
    } else {
      digitalWrite(serialPin, LOW);
    }
    delayMicroseconds(10);
    digitalWrite(clockPin,HIGH);
    delayMicroseconds(10);
    digitalWrite(clockPin,LOW);
    delayMicroseconds(10);
  } //end of loop shifting data out
  digitalWrite(serialPin,LOW);
  delayMicroseconds(10);
  //write latch high to enable parallel latch into register
  digitalWrite(latchPin,HIGH);
  delayMicroseconds(10);
//lower latch to strobe data into output register
  digitalWrite(latchPin,LOW);
  delayMicroseconds(10);
}  
void changeChannel( byte whichChannel)
 {
switch (whichChannel) {
    case 0:     //channel_A
        digitalWrite(channel0,LOW);
        digitalWrite(channel1,LOW);
      break;
    case 1:     //channel_B
        digitalWrite(channel0,HIGH);
        digitalWrite(channel1,LOW);
      break;
    case 2:     //channel_C
        digitalWrite(channel0,LOW);
        digitalWrite(channel1,HIGH);
      break;
    case 3:     //channel_D
        digitalWrite(channel0,HIGH);
        digitalWrite(channel1,HIGH);
      break;
    default: 
      // if nothing else matches, do the default: set to channel_B, OUNBOUND!
        digitalWrite(channel0,HIGH);
        digitalWrite(channel1,LOW);
    break;
  }
 }


